package pacworld;

import agent.AgentRepresentation;
import agent.Action;
import agent.State;

/** A package delivery agent action that causes the agent to pickup a
    package that is located next to the agent in the specified direction.
*/
public class Pickup extends Action {

  public Pickup(int d) {
    if (d < 0 || d > 3)
      System.out.println("ERROR - Invalid direction for Pickup: " + d);
    direction = d;
  }

  /** Return the state that results from the agent moving in the given direction
      in the given state. In order to avoid creating unecessary objects, we
      do not create a new state, but instead modify the old one. This
      would have to change if the Environment needs to maintain a history
      of states. */
  public State execute(AgentRepresentation a, State s) {

    int x,y;
    int pacX, pacY;
    Package p;
    PackageState state = null;
    PacAgentRep agent = null;

    if (s instanceof PackageState)
      state = (PackageState)s;
    else
      System.out.println("ERROR - Argument to Pickup.execute() is not of type PackageState");

    if (a instanceof PacAgentRep)
      agent = (PacAgentRep)a;
    else
      System.out.println("ERROR - Argument to Pickup.execute() is not of type PacAgentRep");

    /* since we will be copying the state and bumps depend only on the
       current move, unset the bump condition */
    state.setBump(agent, false);

    state.setIdleCount(0);
    state.setMessage(agent, null);

    p = agent.getHeldPackage();
    if (p != null) {
      System.out.println("ERROR - Agent passed to Pickup.execute() already has a Package");
      return state;
    }

    x = agent.getX();
    y = agent.getY();
    pacX = x + Direction.DELTA_X[direction];
    pacY = y + Direction.DELTA_Y[direction];
    Object obj = state.getObjectAt(pacX, pacY);
    if (obj instanceof Package)
      state.pickup(agent, (Package)obj);
    return state;
  }

  public String toString() {
    return "PICKUP " + Direction.toString(direction);
  }

  private int direction;
}
