package pacworld;

import agent.AgentRepresentation;
import agent.Action;
import agent.State;

/** A pacage delivery agent action that causes the agent to advance one
    step in the given direction. */
public class Say extends Action {
  private String str;

  public Say(String s) {
    str = s;
  }

  /** Return the state that results from the agent doing nothing.
      In order to avoid creating unecessary objects, we
      do not create a new state, but instead modify the old one. This
      would have to change if the Environment needs to maintain a history
      of states. */
  public State execute(AgentRepresentation a, State s) {
    PackageState state = null;
    PacAgentRep agent = null;

    if (s instanceof PackageState)
      state = (PackageState)s;
    else
      System.out.println("ERROR - Argument to Idle.execute() is not of type PackageState");

    if (a instanceof PacAgentRep)
      agent = (PacAgentRep)a;
    else
      System.out.println("ERROR - Argument to Pickup.execute() is not of type PacAgentRep");

    /* since we will be copying the state and bumps depend only on the
       current move, unset the bump condition */
    state.setBump(agent, false);
    state.setIdleCount(0);
    state.setMessage(agent, str);
    return state;
  }

  public String toString() {
    return "SAY " + str;
  }
}
